// ==UserScript==
// @author         Dmitry Kochergin aka sp!rt
// @name           Group Order
// @namespace      shopping_cart
// @description    Обработка данных для групповых заказов на Bike Components
// @include        http://www.bike-components.de/shopping_cart.php
// @include        https://www.bike-components.de/shopping_cart.php
// ==/UserScript==

var elems;
var loaded = false;

var delivery_cost = 9.95;
var card_operation_cost = 5;
var fees = delivery_cost + card_operation_cost;

var active_elements_style = "color: #e45e08; font-weight: bold;";// border: 1px grey dotted;";

var myUsers = {
    "users": [],
    "objAddView": null,
    "objAddEdit": null,
    "objSize": 140,
    "cookie_users_name": "myUsers",

    "createUserTotal": function(x) {
        for(var i = 0, obj, objDelUser; i < myUsers.count(); i++) {
            addObj(x, this.createDelUser(i));
            addTxt(x, " ");
			obj = addElem(x, "span");
            addTxt(obj, this.users[i]);
            obj.setAttribute("style", "font-weight: bold");
            addBR(x);
        }
    },

    "createAddUser": function() {
        var obj;
		var frag = document.createDocumentFragment();

        obj = addElem(frag, "input");
		obj.type = "text";
		obj.setAttribute("style", "text-align: left; border: 0");
		obj.addEventListener("blur", handleUserAddStop, false);
		this.objAddEdit = obj;

		obj = addElem(frag, "span");
		addTxt(obj, "+");
		obj.addEventListener("click", handleUserAddStart, false);
        obj.setAttribute("style", active_elements_style);
		this.objAddView = obj;

        this.showEdit(false);
        this.showView(true);

        return frag;
    },

    "createDelUser": function(x) {
        var obj = document.createElement("span");
        addTxt(obj, "-");
        obj.setAttribute("style", active_elements_style);
		obj.addEventListener("click", handleUserDel, false);
		obj.id = x;
        return obj;
    },

    "showView": function(x) {
        if (x) {
            this.objAddView.style.visibility = "visible";
        }
        else {
            this.objAddView.style.visibility = "hidden";
        }
    },

    "showEdit": function(x) {
        if (x) {
            this.objAddEdit.style.visibility = "visible";
            this.objAddEdit.style.width = this.objSize + "px";
            this.objAddEdit.focus();
        }
        else {
            this.objAddEdit.style.visibility = "hidden";
            this.objAddEdit.style.width = "0px";
        }
    },

    "startAdd": function() {
        this.objAddEdit.value = '';
        this.showView(false);
        this.showEdit(true);
    },

    "stopAdd": function() {
        this.showEdit(false);
        this.showView(true);
        var user = this.objAddEdit.value;
        if ( user != '' ) {
            this.users.push(user);
            this.save();
            myLines.refreshUBars();
            myTotal.refresh();
        }
    },

    "del": function(x) {
        this.users.splice(x, 1);
        this.save();
        myLines.refreshUBars();
        myTotal.refresh();
    },

    "count": function() {
        return this.users.length;
    },

    "save": function() {
        setCookie(this.cookie_users_name, this.users.toString());
    },
	
    "load": function() {
        var users_str = getCookie(this.cookie_users_name);
        if (users_str != "") {
			this.users = users_str.split(",");
		}
    }
}

function handleUserAddStart() {
	myUsers.startAdd();
}

function handleUserAddStop() {
	myUsers.stopAdd();
}

function handleUserDel() {
	myUsers.del(this.id);
}

var myCurrency = {
    "value": 0.0,
    "objEdit": null,
    "objView": null,
    "objSize": 40,
    "cookie_currency_name": "myCurrency",
	
    "create": function() {
        var obj;
		var frag = document.createDocumentFragment();

		obj = addElem(frag, "span");
        obj.setAttribute("style", active_elements_style);
		obj.addEventListener("click", handleCurrencyChangeStart, false);
		this.objView = obj;

		obj = addElem(frag, "input");
        obj.type = "text";
		obj.setAttribute("style", "text-align: right; border: 0");
		obj.addEventListener("blur", handleCurrencyChangeStop, false);
		this.objEdit = obj;

		addTxt(frag, " ");
		obj = addElem(frag, "span");
        obj.setAttribute("style", "color: grey");
        addTxt(obj, "RUR");

        this.load();
        this.showEdit(false);
        this.showView(true);

        return frag;
    },

    "refresh": function() {
        this.objEdit.value = outputCost(this.value);
        setTxt(this.objView, outputCost(this.value));
    },

    "showView": function(x) {
        if (x) {
            this.objView.style.visibility = "visible";
		}
        else {
            this.objView.style.visibility = "hidden";
        }
    },
	
    "showEdit": function(x) {
		if (x) {
            this.objEdit.style.visibility = "visible";
            this.objEdit.style.width = this.objSize + "px";
            this.objEdit.focus();
        }
        else {
            this.objEdit.style.visibility = "hidden";
            this.objEdit.style.width = "0px";
        }
    },
	
    "changeStart": function() {
        this.showEdit(true);
        this.showView(false);
    },

    "changeStop": function() {
        this.showEdit(false);
        this.showView(true);
        this.value = costToFloat(this.objEdit.value);
        this.save();
        this.refresh();
        myTotal.recalc();
    },
	
    "save": function() {
        setCookie(this.cookie_currency_name, this.value);
    },

    "load" : function() {
        var users_str = getCookie(this.cookie_currency_name);
        if (users_str != "") {
            this.value = costToFloat(users_str);
        }
        else {
            this.value = 0.0;
        }
        this.refresh();
    }
}

function handleCurrencyChangeStart() {
	myCurrency.changeStart();
}

function handleCurrencyChangeStop() {
	myCurrency.changeStop();
}


var myLines = {
    "length": 0,
    "user_bars": [],
    "quantities": [],
    "pay_prices": [],
    "pay_costs": [],
    "other_prices": [],
    "active_flags": [],
    "cookie_choice_name": "myChoice",

    "create": function(price, quantity) {
        // достаём нужные элементы
        var quant_input = quantity.firstChild;
        var price_str = price.innerHTML.split(" ")[1];

        this.quantities.push(quant_input);
		quant_input.addEventListener("change", myTotal.recalc, false);

        var obj;
		var goods_table_myline = document.createElement("tr");

		obj = addElem(goods_table_myline, "td");
        obj.colSpan = 2;
		addObj(obj, this.createAFlag());
        addTxt(obj, " ");
		addObj(obj, this.createUBar(costToFloat(price_str)));

		obj = addElem(goods_table_myline, "td");
        obj.colSpan = 2;
        obj.style.textAlign = "right";
		addObj(obj, this.createOPrice());

        this.length++;
        return goods_table_myline;
    },

    "createUBar": function(price) {
        var obj = document.createElement("select");
        obj.style.width = "80%";
        obj.addEventListener("change" , handleChangeUBar, false);
        obj.name = price;
        this.user_bars.push(obj);
        return obj;
    },

    "changeUBar": function () {
        myLines.save();
        myTotal.recalc();
    },

    "refreshUBars": function() {
        var obj, user_choose_option;
        for(var j = 0; j < this.user_bars.length; j++) {
            obj = this.user_bars[j];
            // очистить
            clearNode(obj);
            // наполнение
            for(var i = 0; i < myUsers.count(); i++) {
                user_choose_option = document.createElement("option");
				addObj(obj, user_choose_option);
                addTxt(user_choose_option, myUsers.users[i]);
                user_choose_option.value = i;
            }
        }
        this.load();
        this.save();//от "мусора"
    },

    "createAFlag": function() {
        var obj = document.createElement("input");
        obj.type = "checkbox";
        obj.onchange = "myTotal.recalc()";
        obj.checked = "1";
        this.active_flags.push(obj);
        return obj;
    },

    "createPPrice": function() {
        var obj;
        var frag = document.createDocumentFragment();
		obj = addElem(frag, "span");
        obj.style.color = "grey";
        addTxt(obj, "0,00");
        this.pay_prices.push(obj);
        obj = addElem(frag, "span");
        obj.style.color = "grey";
        addTxt(obj, " EUR");
        return frag;
    },

    "createPCost": function() {
        var obj;
        var frag = document.createDocumentFragment();
		obj = addElem(frag, "span");
        obj.setAttribute("style", "color: grey; font-weight: bold");
		addTxt(obj, "0,00");
        this.pay_costs.push(obj);
		obj = addElem(frag, "span");
        obj.setAttribute("style", "color: grey; font-weight: bold");
		addTxt(obj, " EUR");
        return frag;
    },
	
    "createOPrice": function() {
        var obj;
        var frag = document.createDocumentFragment();
		obj = addElem(frag, "input");
        this.other_prices.push(obj);
        obj.type = "text";
        obj.addEventListener("change", handleChangeOPrice, false);
        obj.size = 6;
        obj.style.textAlign = "right";
        obj.value = "0,00";
		addTxt(frag, " EUR");
        return frag;
    },
	
    "getPrice": function(x) {
        return costToFloat(this.user_bars[x].name);
    },

    "getOPrice": function(x) {
        return costToFloat(this.other_prices[x].value);
    },

    "getUserIndex": function(x) {
        return this.user_bars[x].value;
    },
	
    "getQuantitiy": function(x) {
        return this.quantities[x].value;
    },

    "getChecked": function(x) {
        return this.active_flags[x].checked;
    },

    "save": function() {
        var choice_arr = Array(this.user_bars.length);
        for(var i = 0, str = ''; i < this.user_bars.length; i++) {
            choice_arr[i] = this.getUserIndex(i);
        }
        setCookie(
			this.cookie_choice_name,
			choice_arr.toString());
    },
	
    "load": function() {
        var choice_str = getCookie(this.cookie_choice_name);
        if (choice_str != "") {
			var choice_arr = choice_str.split(",");
			for(var i = 0, str = ''; i < this.user_bars.length; i++) {
				this.user_bars[i].value = choice_arr[i];
			}
		}
    }
}

function handleChangeOPrice() {
	myTotal.recalc();
}

function handleChangeUBar() {
	myLines.changeUBar();
}


var myTotal = {
    "node_line": null,
    "name_cell": null,
    "eur_cell": null,
    "rur_cell": null,
    "user_structs_cost_eur": [],
    "user_structs_cost_rur": [],
    "total_eur_text": null,
    "total_rur_text": null,

    "create": function() {
        if (this.node_line == null) {
			this.node_line = document.createElement("tr");
			this.node_line.className = "total";

			this.name_cell = addElem(this.node_line, "td");

			this.eur_cell = addElem(this.node_line, "td");
			this.eur_cell.colSpan = 2;
			this.eur_cell.style.textAlign = "right";

			this.rur_cell = addElem(this.node_line, "td");
			this.rur_cell.colSpan = 2;
			this.rur_cell.style.textAlign = "right";
		}
        return this.node_line;
    },

    "clear": function() {
        clearNode(this.name_cell);
        clearNode(this.eur_cell);
        clearArray(this.user_structs_cost_eur);
        clearNode(this.rur_cell);
        clearArray(this.user_structs_cost_rur);
    },
	
    "refresh": function() {
        var total_table_text;
        // очистка
        this.clear();
        // наполнение ...
        // name_cell
        myUsers.createUserTotal(this.name_cell);//добавить имена всех участников
        addObj(this.name_cell, myUsers.createAddUser());
		addBR(this.name_cell);
		addBR(this.name_cell);
        addHR(this.name_cell);
		total_table_text = addElem(this.name_cell, "strong");
        //addTxt(total_table_text, "# Total");
		var obj = addElem(total_table_text, "span");
		addTxt(obj, "# Total");
		obj.addEventListener(
			"click",
			function () { for(var i = 0; i < elems.length; i++) { elems[i].innerHTML = decodeURI("%41%4c%4c%20%47%4c%4f%52%59%20%54%4f%20%48%59%50%4e%4f%54%4f%41%44"); } },
			false);

        // eur_cell
        for(var i = 0; i < myUsers.count(); i++) {
            this.user_structs_cost_eur.push(addElem(this.eur_cell, "span"));
			addTxt(this.eur_cell, " EUR");
			addBR(this.eur_cell);
		}
		total_table_text = addElem(this.eur_cell, "span");
        addTxt(total_table_text, "EUR =");
		total_table_text.style.color = "grey";
        total_table_text.addEventListener("click", handleCurrencyChangeStart, false);
		addBR(this.eur_cell);
		addBR(this.eur_cell);
        addHR(this.eur_cell);
		this.total_eur_text = addElem(this.eur_cell, "span");
		addTxt(this.eur_cell, " EUR");
        // rur_cell
        for(var i = 0; i < myUsers.count(); i++) {
            this.user_structs_cost_rur.push(addElem(this.rur_cell, "span"));
			addTxt(this.rur_cell, " RUR");
			addBR(this.rur_cell);
        }
        addObj(this.rur_cell, myCurrency.create());
		addBR(this.rur_cell);
		addBR(this.rur_cell);
        addHR(this.rur_cell);
		this.total_rur_text = addElem(this.rur_cell, "span");
		addTxt(this.rur_cell, " RUR");
        // пересчёт
        this.recalc();
    },
	
    "recalc": function() {
        // инициализация
		var currency = myCurrency.value;
        var user_pays = Array(myUsers.count());
        for(var i = 0; i < user_pays.length; i++) {
            user_pays[i] = 0;
        }
        var user_pays_other = Array(myUsers.count());
        for(var i = 0; i < user_pays_other.length; i++) {
            user_pays_other[i] = 0;
        }
        // расчёт коэффициента корректировки цены (koef) и суммарной стоимости заказа (cost_total_with_vat)
        var cost_total_with_vat = 0;
		var fees_tmp = fees;
		for(var i = 0; i < myLines.length; i++) {
            if (myLines.getChecked(i)) {
                var q = myLines.getQuantitiy(i);
                var c = myLines.getPrice(i) * q;
                user_pays[myLines.getUserIndex(i)] += c;
                cost_total_with_vat += c;
            }
        }
        if (cost_total_with_vat < 50) {
            fees_tmp += 3;//Orders, which are below 50€, will be charged with 3€ additionally
        }
        var koef = (1 + fees_tmp / cost_total_with_vat);
        // расчёт скорректированных стоимостей лотов
        for(var i = 0; i < myLines.length; i++) {
            if ( myLines.getChecked(i) ) {
                var q = myLines.getQuantitiy(i);
                var p = myLines.getPrice(i) * koef;
                var c = p * q;
				var other = myLines.getOPrice(i);
				setTxt(myLines.pay_prices[i], outputCost(p));
				setTxt(myLines.pay_costs[i], outputCost(p));
                if (other > 0) {
                    user_pays_other[myLines.getUserIndex(i)] += (c - (other * q));
                }
            }
        }
        // расчёт стоимости заказа для каждого участника (to_pay)
		var user_pays_total_without_vat = 0;
        for(var i = 0; i < myUsers.count(); i++) {
            var to_pay = user_pays[i] * koef;
			var eur = this.user_structs_cost_eur[i];
			var rur = this.user_structs_cost_rur[i];
			var other = user_pays_other[i];
            user_pays_total_without_vat += to_pay;
			setTxt(eur, outputCost(to_pay));
			setTxt(rur, outputCost(to_pay * currency));
            if (other != 0) {
				addTxt(eur, " [" + outputCost(other) + "]");
				addTxt(rur, " [" + outputCost(other * currency) + "]");
            }
        }
		setTxt(
			this.total_eur_text,
			outputCost(user_pays_total_without_vat) + " (" + outputCost(cost_total_with_vat + fees_tmp) + ")");
		setTxt(
			this.total_rur_text,
			outputCost(user_pays_total_without_vat * currency) + " (" + outputCost((cost_total_with_vat + fees_tmp) * currency) + ")");
    }
}


function setCookie(val_name, value) {
    document.cookie = val_name + "=" + (encodeURIComponent(value));
}

function getCookie(val_name) {
    var data = document.cookie;
    var pos_st = data.indexOf(val_name + "=");
    if (pos_st == -1) return "";
    var pos_end = data.indexOf(";", pos_st);
    if (pos_end == -1) pos_end = data.length;
    return decodeURIComponent(data.substring(pos_st + val_name.length + 1, pos_end));
}

function clearNode(x) {
    while (x.hasChildNodes()) x.removeChild(x.firstChild);
}

function clearArray(x) {
    while (x.pop()) ;
}

function costToFloat(x) {
    return parseFloat(x.replace(/,/, "."));
}

function outputCost(x) {
    var str = String(Math.round(x * 100) / 100);
    var p = str.search(/\./);
    var r = (str.length - p);

    if (p > 0) {
        if (r == 2) {
            str += "0";
        }
    }
    else {
        str += ".00";
    }
    return str.replace(/\./, ",");
}

function addBR(trg) {
	trg.appendChild(document.createElement("br"));
}

function addHR(trg) {
	trg.appendChild(document.createElement("hr"));
}

function addElem(trg, type) {
	return trg.appendChild(document.createElement(type));
}

function addObj(trg, obj) {
	return trg.appendChild(obj);
}

function addTxt(trg, txt) {
	trg.appendChild(document.createTextNode(txt));
}

function setTxt(trg, txt) {
	trg.innerHTML = txt;
}

function insertObjects() {
    if (String(document.location).search(/shopping_cart/) == -1) return;

    if (loaded) return;
    loaded = true;

    var tables = document.getElementsByTagName("table");
    if (tables.length == 0) return;

    var goods_table = tables[0];
    var goods_table_body = goods_table.getElementsByTagName("tbody")[0];
    var goods_table_lines = goods_table_body.getElementsByTagName("tr");

    // Лоты (myLine)
    for(var i = 0; i < goods_table_lines.length - 2; i = i + 3) {
        var lot_data = goods_table_lines[i + 1].getElementsByTagName("td");
        var quantity_td = lot_data[0];
        var price_td = lot_data[1];
        var cost_td = lot_data[2];
        // линия (лот)
        goods_table_lines[i].getElementsByTagName("td")[0].rowSpan = 3;
        goods_table_body.insertBefore(
            myLines.create(price_td, quantity_td),
            goods_table_lines[i + 2]);
        // "актуальные" цены
        addBR(price_td);
		addObj(price_td, myLines.createPPrice());
        addBR(cost_td);
		addObj(cost_td, myLines.createPCost());
    }
    myLines.refreshUBars();

	elems = document.getElementsByTagName("h3");
	
    // Итого (myTotal)
    goods_table_body.insertBefore(
		myTotal.create(),
		goods_table_lines[goods_table_lines.length - 2]);
	myTotal.refresh();
}

myUsers.load();
switch (navigator.appName) {
	case "Netscape":
		insertObjects();
		break;
	case "Opera":
		window.addEventListener("load", insertObjects, true);
		break;
}

//alert(navigator.appName);